-- form_low_tracks.vhd
-- jamieson olsen 3 jan 2002

-- accepts the left and right phi index from a single block.  
-- also takes the Pt word (valid, pt_sub_bin, curvature) and
-- forms two tracks packaged like this:

-- VCSSSSPPPPPP

-- where V = valid bit
-- 		 C = track curvature
--		 SSSS = Pt Sub Bin
--		 PPPPPP = is the Phi Value (1-44) of the found track.

-- if there is only one track in the block both tracks coming out of this module will be 
-- the same.

-- if there are two tracks in the block both will be reported by this module.

-- if there are three or more tracks in the block, only the outer two are reported.  the 
-- tracks in the middle are LOST FOREVER!

-- ** revision history **

-- 03 Jan 2002: created, debugged.
-- 08 Aug 2002: add low latency output L1muon track bus, see engineering note 2002-08-05.
--				Pt sub bin goes from 4 bits down to three.  For LOWEST design.
-- 23 Mar 2004: loosen up the "same" criteria.  If the difference between the phy values is 
--              less then or equal to _TWO_ H-layer fibers, then consider the tracks to be 
--              the SAME!  L.phi >= R.phi, always.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity form_low_tracks is
	port (
		clock:			in	std_logic;
		phi_index_l:	in	std_logic_vector(4 downto 0);
		phi_index_r:	in	std_logic_vector(4 downto 0);
		phi_base:		in	std_logic_vector(4 downto 0);	-- 0 or 22
		valid:			in	std_logic;
		pt_sub:			in	std_logic_vector(2 downto 0);
		curve:			in	std_logic;

		muon_trk:		out	std_logic_vector(9 downto 0);	-- format VCEePppppp, pack into IOB

		left_track:		out	std_logic_vector(10 downto 0);	-- format VCEeePppppp
		right_track:	out	std_logic_vector(10 downto 0);
		same:			out	std_logic
	);
end form_low_tracks;

architecture form_low_tracks_arch of form_low_tracks is

	-- define registers...

		signal muon_trk_reg:	std_logic_vector(10 downto 0);
		signal left_track_reg:	std_logic_vector(10 downto 0);
		signal right_track_reg:	std_logic_vector(10 downto 0);
		signal same_reg:		std_logic;

	-- define signals...

		signal temp_left_trk:	std_logic_vector(10 downto 0);
		signal temp_right_trk:	std_logic_vector(10 downto 0);
		signal temp_same:		std_logic;
		signal temp_muon_trk:	std_logic_vector(10 downto 0);

	-- define components...

begin

	-- combinatorial stuff...

--		muon_trk(10 downto 0)		<= muon_trk_reg(10 downto 0);

		temp_left_trk(10)			<= valid;
		temp_left_trk(9)			<= curve;
		temp_left_trk(8 downto 6) 	<= pt_sub(2 downto 0);
		temp_left_trk(5 downto 0) 	<= (('0' & phi_index_l(4 downto 0)) + ('0' & phi_base(4 downto 0)) - 1);

		temp_right_trk(10)			<= valid;
		temp_right_trk(9)			<= curve;
		temp_right_trk(8 downto 6) 	<= pt_sub(2 downto 0);
		temp_right_trk(5 downto 0) 	<= (('0' & phi_index_r(4 downto 0)) + ('0' & phi_base(4 downto 0)) - 1);


--      Here is where the "trucation" of similar tracks is done.  If there are two tracks in the block
--		and the phi difference between these tracks is less than or equal to TWO H-layer (doublet) fibers,
--		then consider these tracks to be the SAME track, and assert the SAME signal.  This signal causes
--		the next pipeline stage to discard one of the two tracks.  modified 23 March 2004.

		temp_same 	<= 	'1' when (phi_index_l(4 downto 0)=phi_index_r(4 downto 0)) else				-- L.phi=R.phi
						'1' when ((phi_index_l(4 downto 0)-phi_index_r(4 downto 0))="00001") else	-- L.phi-R.phi=1
						'1' when ((phi_index_l(4 downto 0)-phi_index_r(4 downto 0))="00010") else	-- L.phi-R.phi=2
						'0';

-- 		the old code was more stringent.
--		temp_same 	<= 	'1' when (phi_index_l(4 downto 0)=phi_index_r(4 downto 0)) else 
--						'0';

		left_track(10 downto 0) 		<= left_track_reg(10 downto 0);
		right_track(10 downto 0) 	<= right_track_reg(10 downto 0);

		same	<= same_reg;

		-- NOTE: only the LEFTMOST track in the block is sent on to L1_muon.
		-- if two or more tracks exist per block, only the LEFTMOST will be reported to
		-- L1muon.  CTOC will see both the LEFTMOST and the RIGHTMOST tracks.

		temp_muon_trk(10)			<= valid; 
		temp_muon_trk(9)			<= curve when (valid='1') else '0';
		temp_muon_trk(8 downto 6)	<= pt_sub(2 downto 0) when (valid='1') else "000";
		temp_muon_trk(5 downto 0)	<= (('0' & phi_index_l(4 downto 0)) + ('0' & phi_base(4 downto 0)) - 1) when (valid='1') else "000000";

	-- sequential stuff...

		reg_outputs: process(clock)
		 begin
			if rising_edge(clock) then
				left_track_reg(10 downto 0)  <= temp_left_trk(10 downto 0);
				right_track_reg(10 downto 0) <= temp_right_trk(10 downto 0);
				same_reg <= temp_same;

--				muon_trk_reg(10 downto 0)	 <= temp_muon_trk(10 downto 0);
				muon_trk	 <= temp_left_trk(9 downto 0);

			end if;
		end process;

end form_low_tracks_arch;
