-- cluster_unpacker.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- clusters are presented to DFEA U1 as two busses of 9-bits each.  
-- see my engineering note 2002-01-22A for more info and drawings.

-- purpose of this modules is to accept serialized clusters from a 
-- pair of cluster finders (located in DFEA 'HIGH' track finder FPGA).

-- a) 	count the number of valid clusters.  '00' = 0 clusters, '01'=one cluster
--		'10'=2 clusters, and '11'=3 or more clusters.  this information is fed 
-- 		directly to the L1 formatting engine.

-- b)	correct the cluster centroid value from cluster finder "A", flatten all
--		the clusters into cphix(5..0) outputs and hold them stable for a whole
--		crossing.

-- ** revision history ** 
-- 05 Feb 2002:	  	created
-- 21 Feb 2002:  	add the offset on A, not B.


library ieee;
use ieee.std_logic_1164.all;

entity cluster_unpacker is
	port(
		clock:		in 	std_logic;						-- dfe board clock 53mhz
		reset:		in	std_logic;						-- active low, async.
		cluster_a:	in	std_logic_vector(8 downto 0);	-- output from cluster finder "a"
		cluster_b:	in	std_logic_vector(8 downto 0);	-- output from cluster finder "b"
		start:		in	std_logic;						-- set to indicate the *LAST* cluster
		
		cphi0:		out	std_logic_vector(5 downto 0);	-- v+phi, send to the cluster finders...
		cphi1:		out	std_logic_vector(5 downto 0);
		cphi2:		out	std_logic_vector(5 downto 0);
		cphi3:		out	std_logic_vector(5 downto 0);
		cphi4:		out	std_logic_vector(5 downto 0);
		cphi5:		out	std_logic_vector(5 downto 0);
		cphi6:		out	std_logic_vector(5 downto 0);
		cphi7:		out	std_logic_vector(5 downto 0)
	);
end cluster_unpacker;


architecture cluster_unpacker_arch of cluster_unpacker is

	-- define signals...

	-- define registers...

		signal cphi0_reg:	std_logic_vector(5 downto 0);
		signal cphi1_reg:	std_logic_vector(5 downto 0);
		signal cphi2_reg:	std_logic_vector(5 downto 0);
		signal cphi3_reg:	std_logic_vector(5 downto 0);
		signal cphi4_reg:	std_logic_vector(5 downto 0);
		signal cphi5_reg:	std_logic_vector(5 downto 0);
		signal cphi6_reg:	std_logic_vector(5 downto 0);
		signal cphi7_reg:	std_logic_vector(5 downto 0);

		signal areg0:		std_logic_vector(5 downto 0);
		signal areg1:		std_logic_vector(5 downto 0);
		signal areg2:		std_logic_vector(5 downto 0);
		signal areg3:		std_logic_vector(5 downto 0);

		signal breg0:		std_logic_vector(5 downto 0);
		signal breg1:		std_logic_vector(5 downto 0);
		signal breg2:		std_logic_vector(5 downto 0);
		signal breg3:		std_logic_vector(5 downto 0);

		signal start_reg:	std_logic;

 begin

	-- combinatorial stuff...

		cphi0(5 downto 0)	<= cphi0_reg(5 downto 0);
		cphi1(5 downto 0)	<= cphi1_reg(5 downto 0);
		cphi2(5 downto 0)	<= cphi2_reg(5 downto 0);
		cphi3(5 downto 0)	<= cphi3_reg(5 downto 0);
		cphi4(5 downto 0)	<= cphi4_reg(5 downto 0);
		cphi5(5 downto 0)	<= cphi5_reg(5 downto 0);
		cphi6(5 downto 0)	<= cphi6_reg(5 downto 0);
		cphi7(5 downto 0)	<= cphi7_reg(5 downto 0);

	-- sequential stuff...

--	process(clock, reset)	change to sync reset Aug-06-04
	process(clock)
	 begin

		if rising_edge(clock) then
		 if (reset='0') then

			start_reg <= '0';

			areg0(5 downto 0)	<= "000000";
			areg1(5 downto 0)	<= "000000";
			areg2(5 downto 0)	<= "000000";
			areg3(5 downto 0)	<= "000000";

			breg0(5 downto 0)	<= "000000";
			breg1(5 downto 0)	<= "000000";
			breg2(5 downto 0)	<= "000000";
			breg3(5 downto 0)	<= "000000";

			cphi0_reg(5 downto 0)	<= "000000";
			cphi1_reg(5 downto 0)	<= "000000";
			cphi2_reg(5 downto 0)	<= "000000";
			cphi3_reg(5 downto 0)	<= "000000";
			cphi4_reg(5 downto 0)	<= "000000";
			cphi5_reg(5 downto 0)	<= "000000";
			cphi6_reg(5 downto 0)	<= "000000";
			cphi7_reg(5 downto 0)	<= "000000";
		 else
			start_reg <= start;

			areg0(5 downto 0)	<= cluster_a(8) & "1" & cluster_a(3 downto 0);
			areg1(5 downto 0)	<= areg0(5 downto 0);
			areg2(5 downto 0)	<= areg1(5 downto 0);
			areg3(5 downto 0)	<= areg2(5 downto 0);

			breg0(5 downto 0)	<= cluster_b(8) & "0" & cluster_b(3 downto 0);
			breg1(5 downto 0)	<= breg0(5 downto 0);
			breg2(5 downto 0)	<= breg1(5 downto 0);
			breg3(5 downto 0)	<= breg2(5 downto 0);

			if (start_reg='1') then
				cphi0_reg(5 downto 0)	<= areg3(5 downto 0);	-- first A cluster
				cphi1_reg(5 downto 0)	<= areg2(5 downto 0);
				cphi2_reg(5 downto 0)	<= areg1(5 downto 0);
				cphi3_reg(5 downto 0)	<= areg0(5 downto 0);	-- last A cluster
				cphi4_reg(5 downto 0)	<= breg3(5 downto 0);	-- first B cluster 
				cphi5_reg(5 downto 0)	<= breg2(5 downto 0);
				cphi6_reg(5 downto 0)	<= breg1(5 downto 0);
				cphi7_reg(5 downto 0)	<= breg0(5 downto 0);	-- last B cluster
			end if;

		 end if;
	  end if;
	end process;


end cluster_unpacker_arch;
