-- L1_delay.vhd
-- jamieson olsen <jamieson@fnal.gov>

--	the doublet count is an unsigned 8-bit integer.  It comes from
--	the MEDIUM Pt Track Finder on the STAT[7..0] bus.  The count is 
--	valid when the START bit is set.  This module simply registers it,
--	holds it and delays it appropriately so that it's valid when the 
--	cluster matcher asserts the CNT_VALID output and loads everything
--	into the L1 formatter.

--  In the L1 protocols, the pass and mark (PM) bits = "11" are "Reserved"
--	refer to engineering note 2002-02-08A for details.

-- *** revision history *** 
--
-- 08 Feb 2002:	  	created and functionally simulated.
-- 21 Feb 2002: 	adjust delay to match up
-- 15 May 2002: 	adjust the delay even more: 5 clock periods setup, 2 hold.
--					medium track finder changed so that the doublet count is 
--					stable for 132 and aligned with the START bit.
-- 25 Jun 2002:		set both PM bits in response to first crossing.
-- 08 Jul 2002:		remove the PM bits, delay CR and FX to be inserted into L1 
--					CTOC output.
-- 12 Aug 2002:		doublets shifted in 2 bits at time:
--					                   ______________________________
--					doublets(2):   ___/                              \_______
--					doublets(1:0): xxx< 7..6 >< 5..4 >< 3..2 >< 1..0 >xxxx

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L1_delay is
	port(
		reset:			in 	std_logic;						-- active LOW 
		clock:			in 	std_logic;						-- 53MHz
		fx:				in	std_logic;						-- FIRST_XING, active high
		cr:				in	std_logic;						-- CFT_RESET (aka INIT), active high
		doublets:		in	std_logic_vector(2 downto 0);	-- count of doublets hit

		cr_dly:			out	std_logic;						-- FIRST_XING, delayed
		fx_dly:			out std_logic;						-- CFT_RESET, delayed
		doublets_dly:	out	std_logic_vector(7 downto 0)
	);
end L1_delay;

architecture L1_delay_arch of L1_delay is

	-- define signals...

		

	-- define registers...

		-- these delay registers *should* get packed into SRL16 primatives...

		signal fx_dly_reg:	std_logic_vector(6 downto 0);
		signal cr_dly_reg:	std_logic_vector(6 downto 0);
		signal doublet_reg:	std_logic_vector(23 downto 0);

 begin

	-- combinatorial stuff...

		doublets_dly(7 downto 0)	<= doublet_reg(23 downto 16);
		cr_dly						<= cr_dly_reg(6);
		fx_dly						<= fx_dly_reg(6);

	-- sequential stuff...

--		delay_sr: process(clock, reset)	 	change to sync reset Aug-06-04
		delay_sr: process(clock)
	 	 begin

			if rising_edge(clock) then
			 if (reset='0') then 
	
				cr_dly_reg(6 downto 0)	<= "0000000";
				fx_dly_reg(6 downto 0)	<= "0000000";
				doublet_reg(23 downto 0) <= X"000000";
			 else
				cr_dly_reg(6 downto 1) 	<= cr_dly_reg(5 downto 0);
				cr_dly_reg(0) 			<= cr;

				fx_dly_reg(6 downto 1) 	<= fx_dly_reg(5 downto 0);
				fx_dly_reg(0) 			<= fx;

				if (doublets(2)='1') then
					doublet_reg(7 downto 6) <= doublet_reg(5 downto 4);
					doublet_reg(5 downto 4) <= doublet_reg(3 downto 2);
					doublet_reg(3 downto 2) <= doublet_reg(1 downto 0);
					doublet_reg(1 downto 0) <= doublets(1 downto 0);
				else
					doublet_reg(15 downto  8) <= doublet_reg( 7 downto  0);
					doublet_reg(23 downto 16) <= doublet_reg(15 downto  8);
				end if;

			 end if;
		  end if;
		end process;

end L1_delay_arch;
