-- muon_top.vhd
-- Muon track sort module for DFEA backend FPGA
-- ENG NOTE: 2002-01-14B

-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 18 Feb 1999:		original implementation targeted at a 288 macrocell CPLD.  Used
--					LIFOs, which introduced a scrabling of the tracks.  The 6 highest
-- 					Pt tracks were not necessarily reported.

-- 14 Jan 2002:		complete re-design using FIFOs, not LIFOs.  Tracks are reported
--					in order with no scrambling.  PARITY_ENABLE output is always
--					asserted on the 7th word of the frame.  XMIT_ENABLE is always on.
--					does not care about the SYNC GAPs -- this will need to be added.
--					design takes up 3% XCV300-4 and runs @ 100MHz+

-- 15 Jan 2002:		XMIT_EN output now responds properly to "sync gap" and "cft reset"
--					control signals.  latency on this path is 2 clk cycles, the same as the 
--					track sorter itself.

-- 04 Feb 2002:		expand the track output bus to 16 bits, organized as V..CBbEeeePppppp:
--						V (bit 15)         set if track is valid
--						C (bit 12)         curvature of the track (set if CW -- see eng note 1999-01-05A)
--						Bb (bits 11..10)   Pt bin (max=11, high=10, med=01, low=00)
--						Eeee (bits 9..6)   extended Pt: the higher the number 
--                                         the higher the Pt within the Pt bin. (0 to 7)
--						Pppppp (bits 5..0) phi value of the found track ranges from 1 to 44,
--                                         inclusive

-- 21 Mar 2002:		This is a *FAKE* version, used for L1 Muon testing.  This design looks to
--					see when SYNC_GAP goes away.  That event is number 7.  A max Pt track is 
--					inserted into event number 10 in the first timeslice.  Muon trigger manager
--					should be able to see this and determine the timing offset between the DFE
--					and L1muo systems.

-- 13 May 2002:		merge the MUON_TOP and MUON_FAKE designs into a single design that supports
--					two test modes: SEND_FAKE inserts a maximum Pt track into the 10th crossing
--					(2nd beam crossing after SYNC_GAP).  SEND_PATT sends a known fixed pattern
--					on the muon bus.  This pattern is 0000, 0101, 0202, ... FFFF with transmit
--					enable ON, then the same 256 words with transmit enable OFF.  (in this mode 
--					Parity enable toggles 1/7 like normal) SEND_FAKE and SEND_PATT are turned on 
--					and off	by setting the DFEA sector address to a special value.  
--					*** See eng. note 2002-02-04a for details ***

-- 17 May 2002: 	fix bugs in pattern generation and fake track generation.  Mask the output
--					when no valid tracks are present.  correct the Pt bin enumeration... 00=max, 
--					11=low.  fix output format of the track bus to: V...CBbEeePppppp (somehow this
--					change was lost!)  expand the one-hot sel bus to 4 bits to facilitate 
--					output blanking when no valid data is present.

-- 21 Jun 2002:		add extra valid track output (registered copy of out_reg(15))

-- 08 Aug 2002:		change width of input track busses to 9 for Max, High, and Med (VCEePppppp); 
--					10 for Lowest (VCEeePppppp). single START bit to mark first track.  assume all inputs
--					are UNREGISTERED.

-- 21 Oct 2002:		add reset to all modules

-- 13 May 2003:		Curve bit = 1 means CCW

-- *** SYNC_GAP / XMIT_ENABLE timing ***

-- XMIT_ENABLE must be high or low for an entire crosssing (7 cycles).  The low to high transition should
-- happen as the first track is clocked into the out_reg().  The high to low transition should happen as 
-- one clock cycle after the LAST track is clocked into out_reg().

-- XMIT_ENABLE should be dropped during SYNC_GAPs.  The XMIT_ENABLE transitions should occur on frame
-- boundaries as mentioned above.

-- XMIT_ENABLE should also be dropped when a CFT_RESET is receieved by the DFEA.  The falling edge should
-- occur on the frame boundary as mentioned above.  XMIT_ENABLE should stay low until the CFT_RESET is cleared
-- and only then it should wait a bit longer until it sees the next SYNC_GAP come and go, then it should
-- start up like normal.

-- CR and SG are active high inputs to this design.  They are to be delayed on the DFE motherboard
-- such that the transitions of these signals occur ONLY on frame boundaries. The frame boundaries are determined 
-- by looking at the start bit.  When this bit is set, it marks the first word of the frame.  
-- There are a total of seven words per frame.


library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;
entity muon_top is
	port(
		clock:		in	std_logic;						-- 53Mhz
		sg:			in	std_logic;						-- sync gap, active high; see comments above
		cr:			in	std_logic;						-- CFT system reset
		reset:		in	std_logic;						-- active low board reset

		start:		in	std_logic;						-- set to mark the first track
		max_trk: 	in std_logic_vector( 9 downto 0);	-- VCEePppppp
		high_trk: 	in std_logic_vector( 9 downto 0);	-- VCEePppppp
		med_trk: 	in std_logic_vector( 9 downto 0);	-- VCEePppppp
		low_trk: 	in std_logic_vector(10 downto 0);	-- VCEeePppppp

		muon_noTrk:	in std_logic;						-- disables muon track output
		send_fake:	in	std_logic;						-- active high, send a fake MaxPt track in crossing 10.
		send_patt:	in	std_logic;						-- set to force a known pattern on the output.

		dat: 			out std_logic_vector(15 downto 0);	-- format V...CBbEeePppppp
		val:			out std_logic;						-- valid track found
		xe:			out std_logic;						-- control signal to muon SLDB
		pe:			out std_logic						-- control signal to muon SLDB
    );
end muon_top;


architecture muon_top_arch of muon_top is

	-- define registers...
	
		signal maxa_reg:		std_logic_vector(9 downto 0);	-- VCEePppppp
		signal maxb_reg:		std_logic_vector(9 downto 0);	-- VCEePppppp
		signal out_reg:		std_logic_vector(15 downto 0);	-- V..CBb.EeePppppp
		signal val_reg:		std_logic;
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
		signal sg_reg:			std_logic;
		signal cr_reg:			std_logic;
--		signal start_reg:		std_logic_vector(6 downto 0);
		signal start_reg:		std_logic_vector(5 downto 0);
		signal xmit_en_reg:	std_logic;
		signal count_reg:		std_logic_vector(4 downto 0);		-- holdoff counter for fake track
		signal par_en_reg:	std_logic;							-- push into IOB
		signal patt_reg:		std_logic_vector(8 downto 0);

	-- define signals...

		signal sel:				std_logic_vector( 3 downto 0);
		signal temp_max:		std_logic_vector( 9 downto 0);	-- VCEePppppp
		signal mux_out:		std_logic_vector(15 downto 0);
		signal high_out:		std_logic_vector(10 downto 0);	-- VCEeePppppp
		signal med_out:		std_logic_vector(10 downto 0);	-- VCEeePppppp
		signal low_out:		std_logic_vector(10 downto 0);	-- VCEeePppppp
		signal exp_high_trk:	std_logic_vector(10 downto 0);	-- VCEeePppppp
		signal exp_med_trk:	std_logic_vector(10 downto 0);	-- VCEeePppppp
		signal temp_xmit:		std_logic;

	-- define state machines...

		type fsm_type is (reset_state, sync_gap, xmit, cft_rst, wait4sg);
		signal state: fsm_type;

		type fake_type is (Freset, Fwait4sg, Fsync_gap, Fclear);
		signal fstate: fake_type;

	-- declare components...

		component muon_fe
			port(
				clock:		in	std_logic;
				reset:		in	std_logic;
				start:		in	std_logic;
				din: 			in std_logic_vector(10 downto 0);	-- track format VCEeePppppp
				sel:			in	std_logic;
				dout: 		out std_logic_vector(10 downto 0)
		    );
		end component;

attribute IOB: string;
attribute IOB of pe : signal is "false";
attribute IOB of xe : signal is "false";
begin

	-- combinatorial stuff...

		dat(15 downto 0)		<= out_reg(15 downto 0);
-- out_reg has been moved up to aa_top Aug-16-04
		out_reg	<= patt_reg(7 downto 0) & patt_reg(7 downto 0) when send_patt = '1'
 						else mux_out;

		pe  <= par_en_reg;

		val <= val_reg;

		-- track output format:			valid		  000    curvature     Pt     Ext.Pt                       Phi
		mux_out(15 downto 0)	<= 	( maxb_reg( 9) & "000" & maxb_reg(8) & "00" & '0' & maxb_reg(7 downto 6) & maxb_reg(5 downto 0) ) when sel(3 downto 0)="0001" else
									( high_out(10) & "000" & high_out(9) & "01" & high_out(8 downto 6) &       high_out(5 downto 0) ) when sel(3 downto 0)="0010" else
									(  med_out(10) & "000" &  med_out(9) & "10" & med_out(8 downto 6)  &       med_out(5 downto 0)  ) when sel(3 downto 0)="0100" else
									(  low_out(10) & "000" &  low_out(9) & "11" & low_out(8 downto 6)  &       low_out(5 downto 0)  ) when sel(3 downto 0)="1000" else
									X"0000";									

		sel(3 downto 0)	<= "0000" when (muon_noTrk = '1' or (maxb_reg( 9)='0' and high_out(10)='0' and med_out(10)='0' and low_out(10)='0')) else
									"0001" when (maxb_reg( 9)='1') else
									"0010" when (maxb_reg( 9)='0' and high_out(10)='1') else
									"0100" when (maxb_reg( 9)='0' and high_out(10)='0' and med_out(10)='1') else
									"1000";

		temp_xmit				<= 	'1' when (state=xmit) else '0';

		xe						<= 	xmit_en_reg;

		-- if send_fake is set, insert fake track (Max Pt bin, extended Pt=0, curve=1 (CCW), phi=21) at
		-- the appropriate time.  temp_max format VCEePppppp

-- all input registers for tracks have been removed, insertion should be moved up one clock cycle
-- But since the old count_reg value seams to be one too less, no change is needed
-- Aug- 13-04
		temp_max(9 downto 0)	<= "1100010101" when (count_reg(4 downto 0)="10011" and send_fake='1') else 
									maxa_reg(9 downto 0);


		-- make the high and medium track busses (VCEePppppp) look like the low track bus (VCEeePppppp)
		-- by expanding the extended Pt field to 3 bits, padding with a zero.

			exp_high_trk(10 downto 0) <= high_trk(9 downto 8) & '0' & high_trk(7 downto 0);
			exp_med_trk(10 downto 0)  <= med_trk(9 downto 8)  & '0' & med_trk(7 downto 0);
-- input registers removed to reduce latency	Aug-09-04
			maxa_reg(9 downto 0)	<= max_trk(9 downto 0);
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
				sg_reg					<= sg;
				cr_reg					<= cr;

	-- sequential stuff...
--		process(clock, reset) change to sync reset Aug-06-04
	 process(clock)
		begin
		  if rising_edge(clock) then
			if (reset='0') then
				patt_reg(8 downto 0)	<= "000000000";
-- input registers removed to reduce latency	Aug-09-04
--				maxa_reg(9 downto 0)	<= "0000000000";
				maxb_reg(9 downto 0)	<= "0000000000";
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				start_reg(6 downto 0)	<= "0000000";
				start_reg(5 downto 0)	<= "000000";
-- this register has been moved up to aa_top Aug-16-04
--				out_reg(15 downto 0)	<= X"0000";
				par_en_reg				<= '0';
				val_reg					<= '0';
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				sg_reg					<= '0';
--				cr_reg					<= '0';
			else
-- input registers removed to reduce latency	Aug-09-04
--				maxa_reg(9 downto 0)	<= max_trk(9 downto 0);
				maxb_reg(9 downto 0)	<= temp_max(9 downto 0);
				patt_reg(8 downto 0)	<= patt_reg(8 downto 0) + 1;
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				sg_reg					<= sg;
--				cr_reg					<= cr;
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				start_reg(6 downto 1)	<= start_reg(5 downto 0);
				start_reg(5 downto 1)	<= start_reg(4 downto 0);
				start_reg(0)			<= start;

				if (send_patt='1') then
-- this register has been moved up to aa_top Aug-16-04
--					out_reg(15 downto 0)	<= patt_reg(7 downto 0) & patt_reg(7 downto 0);
					par_en_reg				<= '0';
					val_reg					<= '0';
				else
-- this register has been moved up to aa_top Aug-16-04
--					out_reg(15 downto 0)	<= mux_out(15 downto 0);
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--					par_en_reg				<= start_reg(0);
					par_en_reg				<= start_reg(0);
					val_reg					<= mux_out(15);
				end if;
			end if;
		end if;
	 end process;

--		fsm_proc: process(clock, reset)  change to sync reset Aug-06-04
		fsm_proc: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				state <= reset_state;
				xmit_en_reg <= '0';

			 else
				if (send_patt='1') then
					xmit_en_reg <= patt_reg(8);
				else
					xmit_en_reg <= temp_xmit;
				end if;

				case state is
					when reset_state =>
						state <= sync_gap;
					when sync_gap =>
						if (cr_reg='1') then
							state <= cft_rst;
						elsif (sg_reg='0') then
							state <= xmit;
						else
							state <= sync_gap;
						end if;
					when xmit =>
						if (cr_reg='1') then
							state <= cft_rst;
						elsif (sg_reg='1') then
							state <= sync_gap;
						else
							state <= xmit;
						end if;
					when cft_rst =>
						if (cr_reg='0') then
							state <= wait4sg;
						else
							state <= cft_rst;
						end if;
					when wait4sg =>
						if (cr_reg='1') then
							state <= cft_rst;
						elsif (sg_reg='1') then
							state <= sync_gap;
						else
							state <= wait4sg;
						end if;
					when others =>
						state <= reset_state;
				end case;
			 end if;
		  end if;
		end process;

		-- determine when we come out of sync gap and reset a 5-bit counter then.
		-- the counter always counts up to a max value of 31 and then stays there.

--		fake_proc: process(clock, reset)	  change to sync reset Aug-06-04
		fake_proc: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				fstate 					<= Freset;
				count_reg(4 downto 0)	<= "00000";
			 else
				case fstate is
					when Freset =>
						fstate <= Fwait4sg;
					when Fwait4sg =>
-- start_reg(0) condition is redundant Nov-20-04
--						if (start_reg(0)='1' and sg_reg='1') then
						if (sg_reg='1') then
							fstate <= Fsync_gap;
						else
							fstate <= Fwait4sg;
						end if;
					when Fsync_gap =>
-- start_reg(0) condition is redundant Nov-20-04
--						if (start_reg(0)='1' and sg_reg='0') then
						if (sg_reg='0') then
							fstate <= Fclear;
						else
							fstate <= Fsync_gap;
						end if;
					when Fclear =>
						fstate <= Fwait4sg;
					when others =>
						fstate <= Freset;
				end case;

				if (fstate=Fclear) then
					count_reg(4 downto 0)	<= "00000";
				elsif (count_reg(4 downto 0) /= "11111") then
					count_reg <= count_reg + 1;
				end if;
			 end if;
		  end if;
		end process;

	-- instantiate components...

		high_fe: muon_fe
			port map(
				din(10 downto 0)	=> exp_high_trk(10 downto 0),
				clock				=> clock,
				reset				=> reset,
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				start				=> start_reg(6),
				start				=> start_reg(5),
				sel					=> sel(1),
				dout(10 downto 0)	=> high_out(10 downto 0)
		    );

		med_fe: muon_fe
			port map(
				din(10 downto 0)	=> exp_med_trk(10 downto 0),
				clock				=> clock,
				reset				=> reset,
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				start				=> start_reg(6),
				start				=> start_reg(5),
				sel					=> sel(2),
				dout(10 downto 0)	=> med_out(10 downto 0)
		    );

		low_fe: muon_fe
			port map(
				din(10 downto 0)	=> low_trk(10 downto 0),
				clock				=> clock,
				reset				=> reset,
-- all input registers for tracks have been removed, start delay
-- must be reduced by one everywhere  Aug-13-04
--				start				=> start_reg(6),
				start				=> start_reg(5),
				sel					=> sel(3),
				dout(10 downto 0)	=> low_out(10 downto 0)
		    );
		
end muon_top_arch;
