-- L2_clustram.vhd
-- part of the L2CFT engine track sorter
-- jamieson olsen <jamieson@fnal.gov>
-- 20 Sept 2002

-- must be loaded before L2CFT tracks come flying by.  During L2CFT processing
-- cluster indicies (cnum) are supplied to this module, and it outputs the 
-- cluster centroid and passes that information on to the L2CFT formatter.

-- 	addr	a_cluster_ram		b_cluster_ram
--  0		cluster0			cluster4
--	1   	cluster1			cluster5
--  2       cluster2			cluster6
--  3 	  	cluster3			cluster7

-- A clusters start in the center of the home sector and increase with phi into the next sector
-- B clusters start in the center of the home sector and decrease with phi into the prev sector

-- so, to read out the clusters in order of increasing phi read like this:
-- 7, 6, 5, 4, 0, 1, 2, 3. 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2_clustram is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;

		cluster_a:	in	std_logic_vector(8 downto 0); 	-- format VCccccWww
		cluster_b:	in	std_logic_vector(8 downto 0);  	-- format VCccccWww
		load:		in	std_logic;						-- active high, assert for 4 clocks.

		cnum:		in	std_logic_vector(2 downto 0);

		cvalid:		out	std_logic;
		centroid:	out	std_logic_vector(4 downto 0);
		width:		out std_logic_vector(2 downto 0)
    );
end L2_clustram;


architecture L2_clustram_arch of L2_clustram is

	-- define registers...
	
		signal write_addr:	std_logic_vector(3 downto 0);
	
	-- define signals

		signal read_addr:	std_logic_vector(3 downto 0);
		signal ca_dout:		std_logic_vector(8 downto 0);
		signal cb_dout:		std_logic_vector(8 downto 0);

	-- declare components...

		component ram9x16d
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector(3 downto 0);	-- write port address
				din:		in	std_logic_vector(8 downto 0);
				addrb:		in	std_logic_vector(3 downto 0);	-- read port address
				dout:		out	std_logic_vector(8 downto 0)	-- read data
			);
		end component;

		
 begin

	-- combinatorial stuff...

		cvalid <= 	ca_dout(8) when (cnum(2)='0') else 
					cb_dout(8);

		centroid(4 downto 0) <= ca_dout(7 downto 3) when (cnum(2)='0') else 
								cb_dout(7 downto 3);

		width(2 downto 0) <= 	ca_dout(2 downto 0) when (cnum(2)='0') else 
								cb_dout(2 downto 0);

		read_addr(3 downto 0) <= "00" & cnum(1 downto 0);

	-- sequential stuff...

--		process(clock, reset)  change to sync reset Aug-06-04
		process(clock)
 		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				write_addr(3 downto 0) <= "0000";
			 else
				if (load='1') then
					write_addr(3 downto 0) <= write_addr(3 downto 0) + 1;
				else
					write_addr(3 downto 0) <= "0000";
				end if;
			 end if;
		  end if;
		end process;
	
	-- instantiate components...

		a_cluster_ram: ram9x16d
		 	port map(
				wclk		      => clock,
				we				  => load,
				addra(3 downto 0) => write_addr(3 downto 0),
				din(8 downto 0)	  => cluster_a(8 downto 0),
				addrb(3 downto 0) => read_addr(3 downto 0),
				dout(8 downto 0)  => ca_dout(8 downto 0)
			);

		b_cluster_ram: ram9x16d
		 	port map(
				wclk		      => clock,
				we				  => load,
				addra(3 downto 0) => write_addr(3 downto 0),
				din(8 downto 0)	  => cluster_b(8 downto 0),
				addrb(3 downto 0) => read_addr(3 downto 0),
				dout(8 downto 0)  => cb_dout(8 downto 0)
			);

end L2_clustram_arch;

