-- L2_engine.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- tie together all of the L2 engine modules and sequence the control signals.  start is asserted
-- for one clock cycle, and marks the first word clocked out of the L2_pipeline.  this start
-- bit starts a counter, which sequences the control signals for all of the modules.

-- *** revision history ***
-- 26 Sept 2002: created, functionally simulated.
-- 23 Oct  2002: added fake record support
--  6 Mar  2003: extend fake L2 select bits to 2.
-- 13 May 2003:  consistant curve 1 = CCW
-- 29 May 2003:	 trim the trksort enable signal down from 7 to 6 clocks, just to prevent any
--				 and junk from being inserted into the tracksorter module.
-- 3/10/2005 flush at the same time as being read out S. X. Wu
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2_engine is
	port(
		reset:		in	std_logic;	  					-- active low
		clock:		in	std_logic;	  					-- 53MHz
		start:		in  std_logic;	  					-- mark the first word being clocked in
		send_fake:	in	std_logic_vector( 1 downto 0);
		din:		in	std_logic_vector(79 downto 0);	-- from the pipeline
		sector:		in	std_logic_vector( 3 downto 0);	-- relative sector 0-15
		octant:		in	std_logic_vector( 2 downto 0); 	-- octant number 0-7

		cft_frame:	out	std_logic_vector(27 downto 0);
		cps_frame:	out	std_logic_vector(27 downto 0)
	);
end L2_engine;

architecture L2_engine_arch of L2_engine is

	-- declare registers...

		signal crossing_reg:	std_logic_vector( 7 downto 0);
		signal turn_reg:		std_logic_vector(15 downto 0);
		signal iso_reg:			std_logic;
		signal pm_reg:			std_logic_vector(1 downto 0);
		signal count_reg:		std_logic_vector(5 downto 0);	-- master sequence control counter

		signal nmax_reg:		std_logic_vector(2 downto 0);	-- number of max tracks 0-6
		signal nhigh_reg:		std_logic_vector(2 downto 0);	-- number of high tracks 0-6
		signal nmed_reg:		std_logic_vector(2 downto 0);	-- number of med tracks 0-6
		signal nlow_reg:		std_logic_vector(2 downto 0);	-- number of low tracks 0-6

	-- declare signals...

		signal number_of_tracks:	std_logic_vector(4 downto 0);	-- total number of CFT tracks
		signal number_of_clusters:	std_logic_vector(3 downto 0);	-- total number of clusters that will be reported

		signal enable_trksort:		std_logic;

		signal flush_track_ram:		std_logic;

		signal cnum:				std_logic_vector(2 downto 0);
		signal cluster_number:		std_logic_vector(2 downto 0);
		signal cft_frame_sel:		std_logic_vector(1 downto 0);
		signal cps_frame_sel:		std_logic_vector(1 downto 0);

		signal valid_track:			std_logic;
		signal pt:					std_logic_vector(1 downto 0);
		signal curve:				std_logic;
		signal cluster_match:		std_logic;
		signal extended_pt:			std_logic_vector(2 downto 0);
		signal phi:					std_logic_vector(5 downto 0);

		signal load_clustram:		std_logic;
		signal valid_cluster:		std_logic;
		signal centroid:			std_logic_vector(4 downto 0);
		signal width:				std_logic_vector(2 downto 0);


		signal start_l2_clustsort:	std_logic;
		signal tr_valid_track:		std_logic;
		signal tr_pt:				std_logic_vector(1 downto 0);
		signal tr_curve:			std_logic;
		signal tr_cluster_match:	std_logic;
		signal tr_extpt:			std_logic_vector(2 downto 0);

		-- signal sorted_valid_cluster:		std_logic;
		signal sorted_centroid:				std_logic_vector(4 downto 0);
		signal sorted_width:				std_logic_vector(2 downto 0);
		-- signal sorted_valid_track:			std_logic;
		signal sorted_pt:					std_logic_vector(1 downto 0);
		signal sorted_curve:				std_logic;
		signal sorted_cluster_match:		std_logic;
		signal sorted_extended_pt:			std_logic_vector(2 downto 0);

		signal fake_L2CFT:		std_logic;
		signal fake_L2CPS:		std_logic;

	-- declare components...

		component L2_trksort
			port(
				clock:	in	std_logic;
				reset:	in	std_logic;
				enable:	in	std_logic;
				max:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
				high:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
				med:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
				low:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
				tvalid:	out	std_logic;						-- valid track
				curve:	out	std_logic;		
				pt:		out	std_logic_vector(1 downto 0);
				extpt:	out	std_logic_vector(2 downto 0);	-- extended pt
				phi:	out	std_logic_vector(5 downto 0);	
				cm:		out	std_logic;						-- there is a cluster match
				cnum:	out	std_logic_vector(2 downto 0)	-- matching cluster index
		    );
		end component;

		component L2_clustsort
			port(
				clock:			in	std_logic;
				reset:			in	std_logic;
				start:			in	std_logic;			-- set to mark first data coming in
				tvalid:			in	std_logic;						-- valid track
				curve:			in	std_logic;		
				cm:				in	std_logic;						-- cluster match
				pt:				in	std_logic_vector(1 downto 0);
				extpt:			in	std_logic_vector(2 downto 0);	-- extended pt
				cvalid:			in	std_logic;
				centroid:		in	std_logic_vector(4 downto 0);	
				width:			in	std_logic_vector(2 downto 0);
				cluster_count:	out	std_logic_vector(3 downto 0);	-- number of clusters to be reported
				frame_sel:		out	std_logic_vector(1 downto 0);	-- tell L2CPS_FORMAT what frames to send.
				tvalid_out:		out	std_logic;						-- valid track
				curve_out:		out	std_logic;		
				cm_out:			out	std_logic;						-- cluster match
				pt_out:			out	std_logic_vector(1 downto 0);
				extpt_out:		out	std_logic_vector(2 downto 0);	-- extended pt
				cvalid_out:		out	std_logic;
				centroid_out:	out	std_logic_vector(4 downto 0);	
				width_out:		out	std_logic_vector(2 downto 0)
		    );
		end component;

		component L2_clustram
			port(
				clock:		in	std_logic;
				reset:		in	std_logic;
				cluster_a:	in	std_logic_vector(8 downto 0); 	-- format VCccccWww
				cluster_b:	in	std_logic_vector(8 downto 0);  	-- format VCccccWww
				load:		in	std_logic;						-- active high, assert for 4 clocks.
				cnum:		in	std_logic_vector(2 downto 0);
				cvalid:		out	std_logic;
				centroid:	out	std_logic_vector(4 downto 0);
				width:		out std_logic_vector(2 downto 0)
		    );
		end component;

		component L2_trackram
			port(
				clock:		in	std_logic;
				reset:		in	std_logic;
				tvalid:		in	std_logic;						-- valid track
				curve:		in	std_logic;						-- track curvature 1 = CCW
				cm:			in	std_logic;						-- track is matched to a cluster
				pt:			in	std_logic_vector(1 downto 0);	-- pt bin
				extpt:		in	std_logic_vector(2 downto 0);	-- extended pt info
				cnum:		in	std_logic_vector(2 downto 0);	-- cluster index
				flush:		in	std_logic;						-- empty out the ram
				tvalid_out:	out	std_logic;
				cm_out:		out	std_logic;
				curve_out:	out	std_logic;
				pt_out:		out	std_logic_vector(1 downto 0);
				extpt_out:	out	std_logic_vector(2 downto 0)
		    );
		end component;

		component L2CPS_format
			port(
				reset:			in	std_logic;	  					-- active low, async.
				clock:			in	std_logic;						-- 53MHz
				send_fake:		in	std_logic;						-- send fake L2CPS record instead
				frame_sel:		in	std_logic_vector( 1 downto 0);	-- select the type of word to be output	
				pm:				in	std_logic_vector( 1 downto 0);	-- pass and mark bits
				octant:			in	std_logic_vector( 2 downto 0);	-- octant number (0-7)
				sector:			in	std_logic_vector( 3 downto 0);	-- relative sector within an octant (0-9)
				turn:			in	std_logic_vector(15 downto 0);	-- turn counter
				cross:			in	std_logic_vector( 7 downto 0);	-- crossing counter
				iso:			in	std_logic;						-- a single isolated track
				cluster_count:	in	std_logic_vector(3 downto 0);	-- number of found clusters to be reported for L2CPS [0-8]	
				--cvalid:			in	std_logic;						-- cluster data from the L2engine
				centroid:		in	std_logic_vector(4 downto 0);
				width:			in	std_logic_vector(2 downto 0);
				--tvalid:			in	std_logic;						-- track data from the L2engine
				curve:			in	std_logic;
				pt:				in	std_logic_vector(1 downto 0);
				extpt:			in 	std_logic_vector(2 downto 0);
				cm:				in	std_logic;
				dout:			out	std_logic_vector(27 downto 0)	-- record output, parity has NOT been addeded, yet.
			);
		end component;

		component L2CFT_format
			port(
				reset:		in	std_logic;	  					-- active low, async.
				clock:		in	std_logic;						-- 53MHz
				send_fake:	in	std_logic;						-- send fake L2CFT record instead
				frame_sel:	in	std_logic_vector( 1 downto 0);	-- select the type of word to be output
				pm:			in	std_logic_vector( 1 downto 0);	-- pass and mark bits
				octant:		in	std_logic_vector( 2 downto 0);	-- octant number (0-7)
				sector:		in	std_logic_vector( 3 downto 0);	-- relative sector within an octant (0-9)
				turn:		in	std_logic_vector(15 downto 0);	-- turn counter
				cross:		in	std_logic_vector( 7 downto 0);	-- crossing counter
				iso:		in	std_logic;						-- a single isolated track
				track_cnt:	in	std_logic_vector( 4 downto 0);	-- number of found tracks to be reported for L2CFT [0-24]
				cvalid:		in	std_logic;						-- cluster data from the L2engine
				centroid:	in	std_logic_vector(4 downto 0);
				tvalid:		in	std_logic;						-- track data from the L2engine
				curve:		in	std_logic;
				pt:			in	std_logic_vector(1 downto 0);
				extpt:		in 	std_logic_vector(2 downto 0);
				phi:		in 	std_logic_vector(5 downto 0);
				cm:			in	std_logic;
				dout:		out	std_logic_vector(27 downto 0)	-- record output, parity has NOT been addeded, yet.
			);
		end component;

begin

	-- combinatorial stuff...

		-- add up the number of tracks in each pt bin

			number_of_tracks(4 downto 0) <= ('0' & (('0' & nmax_reg(2 downto 0)) + ('0' & nhigh_reg(2 downto 0)))) +
											('0' & (('0' & nmed_reg(2 downto 0)) + ('0' & nlow_reg(2 downto 0))));

		-- assert enable_trksort while tracks are being clocked into L2_trksort

			enable_trksort <= 	'1' when (count_reg(5 downto 0)="000000") else 
								'1' when (count_reg(5 downto 0)="000001") else 
								'1' when (count_reg(5 downto 0)="000010") else 
								'1' when (count_reg(5 downto 0)="000011") else 
								'1' when (count_reg(5 downto 0)="000100") else 
								'1' when (count_reg(5 downto 0)="000101") else 
								-- '1' when (count_reg(5 downto 0)="000110") else  -- 29 August 2003
								'0';

		-- L2_trksort requires its input bus looks like this: VCEeeePpppppMNnn
		-- but this isn't the way it comes out of the L2_pipeline.  move the fields around.

		-- determine when to load clusters into the cluster ram module.

			load_clustram <= 	'1' when (start='1') else 
								'1' when (count_reg(5 downto 0)="000000") else 
								'1' when (count_reg(5 downto 0)="000001") else 
								'1' when (count_reg(5 downto 0)="000010") else
								'0';
		-- after the L2CPS cycle is complete, completely flush out the trackram
		-- so that old track data does not corrupt the next L2 readout.

--			flush_track_ram <= 	'1' when (count_reg(5 downto 0)="110010") else 
--								'1' when (count_reg(5 downto 0)="110011") else
--								'1' when (count_reg(5 downto 0)="110100") else
--								'1' when (count_reg(5 downto 0)="110101") else
--								'1' when (count_reg(5 downto 0)="110110") else
--								'1' when (count_reg(5 downto 0)="110111") else
--								'1' when (count_reg(5 downto 0)="111000") else
--								'1' when (count_reg(5 downto 0)="111001") else
			flush_track_ram <= 	'1' when (count_reg(5 downto 0)="011100") else 
								'1' when (count_reg(5 downto 0)="011101") else
								'1' when (count_reg(5 downto 0)="011110") else
								'1' when (count_reg(5 downto 0)="011111") else
								'1' when (count_reg(5 downto 0)="100000") else
								'1' when (count_reg(5 downto 0)="100001") else
								'1' when (count_reg(5 downto 0)="100010") else
								'1' when (count_reg(5 downto 0)="100011") else
								'0';

		-- CNUM is applied to both the L2_clustram and the L2_trackram modules.  during
		-- L2CFT time CNUM is driven by the L2_trksort module.  However at L2CPS time
		-- CNUM must be cycled 7, 6, 5, 4, 0, 1, 2, 3.  The clusters are then presented to 
		-- the L2_clustsort module in order of increasing phi.

			cnum(2 downto 0) <= "111" when (count_reg(5 downto 0)="011100") else 
								"110" when (count_reg(5 downto 0)="011101") else 
								"101" when (count_reg(5 downto 0)="011110") else 
								"100" when (count_reg(5 downto 0)="011111") else 
								"000" when (count_reg(5 downto 0)="100000") else 
								"001" when (count_reg(5 downto 0)="100001") else 
								"010" when (count_reg(5 downto 0)="100010") else 
								"011" when (count_reg(5 downto 0)="100011") else
								cluster_number(2 downto 0);

		-- cft_frame_sel tells the L2CFT formatter what type of frame should be generated
		-- (header, turn&crossing, or data).  Must be timed into the tracks as they come
		-- flying out of the L2_trksort module.  assume 24 tracks are reported.

			cft_frame_sel(1 downto 0) <= "01" when (count_reg(5 downto 0)="000001") else	-- header frame
										 "10" when (count_reg(5 downto 0)="000010") else	-- turn-n-crossing frame
										 "11" when (count_reg(5 downto 0)>"000010" and count_reg(5 downto 0)<"011011") else	-- cft track frame
										 "00";

		-- start_l2_clustsort is asserted at the beginning of the L2CPS cycle.  it marks the first
		-- cluster/track data word being clocked into L2_clustsort.

			start_l2_clustsort <= '1' when (count_reg(5 downto 0)="011100") else '0'; 


		-- based on the fake input bits, choose if the L2CFT and L2CPS formatters will send
		-- fake events in response to a L1_ACCEPT...

			fake_L2CFT <= '1' when send_fake(1 downto 0)="10" else '0';
			fake_L2CPS <= '1' when send_fake(1 downto 0)="11" else '0';
	



	-- sequential stuff...

--		process(clock, reset) change to sync reset Aug-06-04
		process(clock)
		 begin

			if rising_edge(clock) then
			 if (reset='0') then
				count_reg(5 downto 0) 		<= "111111";
				crossing_reg(7 downto 0) 	<= X"00";
				turn_reg(15 downto 0)		<= X"0000";
				pm_reg(1 downto 0)			<= "00";
				iso_reg						<= '0';
				nmax_reg(2 downto 0)		<= "000";
				nhigh_reg(2 downto 0)		<= "000";
				nmed_reg(2 downto 0)		<= "000";
				nlow_reg(2 downto 0)		<= "000";
			 else
				if (start='1') then									-- load the static stuff
					crossing_reg(7 downto 0) 	<= din(58 downto 51);
					turn_reg(15 downto 0)		<= din(50 downto 35);
					pm_reg(1 downto 0)			<= din(60 downto 59);
					iso_reg						<= din(61);

					nmax_reg(2 downto 0)		<= din(79 downto 77);
					nhigh_reg(2 downto 0)		<= din(76 downto 74);
					nmed_reg(2 downto 0)		<= din(73 downto 71);
					nlow_reg(2 downto 0)		<= din(70 downto 68);					
				end if;

				if (start='1') then
					count_reg(5 downto 0) <= "000000";
				elsif (count_reg(5 downto 0) /= "111111") then
					count_reg(5 downto 0) <= count_reg(5 downto 0) + 1;
				end if;
			 end if;
		  end if;
		end process;

	-- instantiate components...

		L2_track_sort: L2_trksort 
			port map(
				clock				=> clock,
				reset				=> reset,
				enable				=> enable_trksort,
				max(14 downto 0)	=> din(79 downto 65),
				high(14 downto 0)	=> din(64 downto 50),
				med(14 downto 0)	=> din(49 downto 35),
				low(14 downto 0)	=> din(34 downto 20),
				tvalid				=> valid_track,
				curve				=> curve,
				pt(1 downto 0)		=> pt(1 downto 0),
				extpt(2 downto 0)	=> extended_pt(2 downto 0),
				phi(5 downto 0)		=> phi(5 downto 0),
				cm					=> cluster_match,
				cnum(2 downto 0)	=> cluster_number
		    );

		
		-- L2_clustram must be loaded with the 8 clusters before tracks start coming out of
		-- L2_trksort.  It operates like a table lookup: given a cluster index, outputs the 
		-- cluster centroid, width, and if the cluster is valid.
		
		cluster_ram: L2_clustram
			port map(
				clock				  => clock,
				reset				  => reset,
				cluster_a(8 downto 0) => din(18 downto 10),		-- clusters 0, 1, 2, and 3
				cluster_b(8 downto 0) => din(9 downto 1),		-- clusters 4, 5, 6, and 7
				load				  => load_clustram,
				cnum(2 downto 0)	  => cnum(2 downto 0),
				cvalid				  => valid_cluster,
				centroid(4 downto 0)  => centroid(4 downto 0),
				width(2 downto 0)	  => width(2 downto 0)
		    );

		-- L2_trackram captures track information as it flows out of L2_trksort.
		-- if a track has a valid cluster match, and no other track has matched 
		-- to the cluster yet, then that track information is stored for later
		-- readout when the L2CPS record is formed.

		track_ram: L2_trackram
			port map(
				clock					=> clock,
				reset					=> reset,
				tvalid					=> valid_track,
				curve					=> curve,
				cm						=> cluster_match,
				pt(1 downto 0)			=> pt(1 downto 0),
				extpt(2 downto 0)		=> extended_pt(2 downto 0),
				cnum(2 downto 0)		=> cnum(2 downto 0),
				flush					=> flush_track_ram,
				tvalid_out				=> tr_valid_track,
				cm_out					=> tr_cluster_match,
				curve_out				=> tr_curve,
				pt_out(1 downto 0) 		=> tr_pt(1 downto 0),
				extpt_out(2 downto 0) 	=> tr_extpt(2 downto 0)
		    );

		CFT_formatter: L2CFT_format
			port map(
				clock					=> clock,
				reset					=> reset,
				send_fake				=> fake_L2CFT,
				frame_sel(1 downto 0)	=> cft_frame_sel(1 downto 0),
				pm(1 downto 0)			=> pm_reg(1 downto 0),
				octant(2 downto 0)		=> octant(2 downto 0),
				sector(3 downto 0)		=> sector(3 downto 0),
				turn(15 downto 0)		=> turn_reg(15 downto 0),
				cross(7 downto 0)		=> crossing_reg(7 downto 0),
				iso						=> iso_reg,
				track_cnt(4 downto 0)	=> number_of_tracks(4 downto 0),
				cvalid					=> valid_cluster,
				centroid(4 downto 0)	=> centroid(4 downto 0),
				tvalid					=> valid_track,
				curve					=> curve,
				pt(1 downto 0)			=> pt(1 downto 0),
				extpt(2 downto 0)		=> extended_pt(2 downto 0),
				phi(5 downto 0)			=> phi(5 downto 0),
				cm						=> cluster_match,
				dout(27 downto 0)		=> cft_frame(27 downto 0)
			);

		cluster_sorter: L2_clustsort
			port map(
				clock						=> clock,
				reset						=> reset,
				start						=> start_L2_clustsort,
				tvalid						=> tr_valid_track,
				curve						=> tr_curve,
				cm							=> tr_cluster_match,
				pt(1 downto 0)				=> tr_pt(1 downto 0),
				extpt(2 downto 0)			=> tr_extpt(2 downto 0),
				cvalid						=> valid_cluster,
				centroid(4 downto 0)		=> centroid(4 downto 0),
				width(2 downto 0)			=> width(2 downto 0),
				cluster_count(3 downto 0)	=> number_of_clusters(3 downto 0),
				frame_sel(1 downto 0)		=> cps_frame_sel(1 downto 0),
				-- tvalid_out					=> sorted_valid_track,
				curve_out					=> sorted_curve,
				cm_out						=> sorted_cluster_match,
				pt_out(1 downto 0)			=> sorted_pt(1 downto 0),
				extpt_out(2 downto 0)		=> sorted_extended_pt(2 downto 0),
				-- cvalid_out					=> sorted_valid_cluster,
				centroid_out(4 downto 0)	=> sorted_centroid(4 downto 0),
				width_out(2 downto 0)		=> sorted_width(2 downto 0)
		    );


		CPS_formatter: L2CPS_format
			port map(
				clock						=> clock,
				reset						=> reset,
				send_fake					=> fake_L2CPS,
				frame_sel(1 downto 0)		=> cps_frame_sel(1 downto 0),
				pm(1 downto 0)				=> pm_reg(1 downto 0),
				octant(2 downto 0)			=> octant(2 downto 0),
				sector(3 downto 0)			=> sector(3 downto 0),
				turn(15 downto 0)			=> turn_reg(15 downto 0),
				cross(7 downto 0)			=> crossing_reg(7 downto 0),
				iso							=> iso_reg,
				cluster_count(3 downto 0)	=> number_of_clusters(3 downto 0),
				-- cvalid						=> sorted_valid_cluster,
				centroid(4 downto 0)		=> sorted_centroid(4 downto 0),
				width(2 downto 0)			=> sorted_width(2 downto 0),
				-- tvalid						=> sorted_valid_track,
				curve						=> sorted_curve,
				pt(1 downto 0)				=> sorted_pt(1 downto 0),
				extpt(2 downto 0)			=> sorted_extended_pt(2 downto 0),
				cm							=> sorted_cluster_match,
				dout(27 downto 0)			=> cps_frame(27 downto 0)
			);

end L2_engine_arch;
