-- L2_top.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- accepts raw L1 data continously.  the data is packed, stored in a pipeline.
-- if a L1_accept is observed then the L2_engine grabs the event as it falls 
-- out of the pipeline and uses it to generate a L2_CFT record and L2_CPS record.

-- ties together the L2_packer, L2_pipeline, and L2_engine (where all the work
-- gets done).  Adds necessary control circuitry to provide a control signal to
-- the L2_packer and L2_engine.  Also instantiates the crossing_count module to
-- keep track of the turn and crossing numbers.

-- control bits CR and FX must be aligned with the start bit and be 
-- stable for 132ns.  L1acc is special, it leads the start bit by 1 clock
-- cycle.

-- *** revision history *** 
--
-- 13 May 2002:	  	sends a *FAKE* L2 CFT record in response to a L1_ACC
--					signal.  after L2CFT transmission switch back over to L1 
--					transmission at the appropriate time.
-- 
-- 09 Sep 2002:		add real turn and crossing counter to canned L2 record.
--
-- 15 Oct 2002:		completely overhauled to support new pipelined L2_engine
--					architecture.  build in L2_packer and L2_pipeline.
-- 					P&M bits are not implemented (yet).
-- 
-- 22 Oct 2002:		fix cluster translation bug.
-- 23 Oct 2002:		add support for fake L2 records.
--  7 Mar 2003:		add locked output
-- 12 Mar 2003:		replace crossing_count module with tick_turn module, the same one used on 
--					L1CTOC and CTTT.  Add bad_fx output.


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity l2_top is
	port(
		clock:		in 	std_logic;	-- 53 MHz clock
		reset:		in 	std_logic;	-- active low board reset

		L1acc:		in	std_logic;	-- L1_ACCEPT, active high
		cr:			in	std_logic;	-- CFT_RESET, active high
		fx:			in	std_logic;	-- FIRST_CROSSING, active high

		octant:		in	std_logic_vector(2 downto 0); -- octant number(0-7)
		sector:		in	std_logic_vector(3 downto 0); -- relative sector within an octant (0-9)

--		depth:				in	std_logic_vector( 5 downto 0);	-- pipeline depth, in 132ns crossings.
-- this avoids wrap around problem Nov-2-2004
		depth:				in	std_logic_vector( 8 downto 0);	-- pipeline depth, in 19ns crossings.

		send_fake:	in	std_logic_vector(1 downto 0); -- set to set fake L2 records after L1ACCEPT

		-- L1 data from various modules...

		start:			in	std_logic;						-- set to mark the first cluster coming in.

		cluster_a:		in	std_logic_vector(8 downto 0);	-- clusters 0-3 format VWwwwCccc
		cluster_b:		in	std_logic_vector(8 downto 0);	-- clusters 4-7 format VWwwwCccc

		max_trk:		in	std_logic_vector(10 downto 0);	-- format VCEeePppppp
		high_trk:		in	std_logic_vector(10 downto 0);
		med_trk:		in	std_logic_vector(10 downto 0);
		low_trk:		in	std_logic_vector(10 downto 0);

		max_cluster:	in	std_logic_vector(3 downto 0);	-- format cluster_match + cluster_number(2..0)
		high_cluster:	in	std_logic_vector(3 downto 0);	
		med_cluster:	in	std_logic_vector(3 downto 0);
		low_cluster:	in	std_logic_vector(3 downto 0);

		iso_track:		in	std_logic;						-- set if there is an isolated track
		L2_noTrk:		in	std_logic;						-- disables L2 track output
		pm:				in	std_logic_vector(1 downto 0);

		locked:			out	std_logic;
		bad_fx:			out	std_logic;
		cft_frame:		out	std_logic_vector(27 downto 0);	
		cps_frame:		out	std_logic_vector(27 downto 0)
	);
end l2_top;

architecture l2_top_arch of l2_top is

	-- define registers...

	-- define signals...

		signal turn_number:		std_logic_vector(15 downto 0);
		signal crossing_number:	std_logic_vector( 7 downto 0);
		signal packer_dout:		std_logic_vector(79 downto 0);
		signal pipeline_dout:	std_logic_vector(79 downto 0);

		signal packer_bor:		std_logic;
		signal start_engine:	std_logic;

		signal cr_delay:		std_logic_vector(7 downto 0);
		signal fx_delay:		std_logic_vector(7 downto 0);
		signal start_delay:		std_logic_vector(7 downto 0);

--		signal actual_depth:	std_logic_vector(5 downto 0);

	-- declare components...

		component tick_turn is
		    port (
				bor:	in	std_logic;	-- marks the first frame of a record (could be L1 or L2)
				l1:		in	std_logic;	-- set if the BOR is for a L1 record
				fx:		in 	std_logic;	-- First Crossing
				cr:		in	std_logic;	-- CFT_RESET aka SCL_INIT 
				clock:	in 	std_logic;	-- 53MHz
				reset:	in	std_logic;	-- active low
				tick:	out	std_logic_vector( 7 downto 0);
				turn:	out	std_logic_vector(15 downto 0);
				bad_fx:	out	std_logic;		-- pulsed if FX comes at the wrong time.
				lock:	out	std_logic		-- set if 
		    );
		end component;

		component L2_packer 
			port(
				clock:			in	std_logic;
				reset:			in	std_logic;
				start:			in	std_logic;
				L2_noTrk:		in	std_logic;		-- disables L2 track output
				cluster_a:		in	std_logic_vector(8 downto 0);
				cluster_b:		in	std_logic_vector(8 downto 0);
				crossing:		in	std_logic_vector(7 downto 0);
				turn:			in	std_logic_vector(15 downto 0);
				pm:				in	std_logic_vector(1 downto 0);
				iso_track:		in	std_logic;
				max_trk:		in	std_logic_vector(10 downto 0);
				high_trk:		in	std_logic_vector(10 downto 0);
				med_trk:		in	std_logic_vector(10 downto 0);
				low_trk:		in	std_logic_vector(10 downto 0);
				max_cluster:	in	std_logic_vector(3 downto 0);
				high_cluster:	in	std_logic_vector(3 downto 0);
				med_cluster:	in	std_logic_vector(3 downto 0);
				low_cluster:	in	std_logic_vector(3 downto 0);
				bor:			out	std_logic;
				dout:			out	std_logic_vector(79 downto 0)
			);
		end component;

		component L2_pipeline
			port(
				reset:	in	std_logic;
				clock:	in	std_logic;
				bor:	in	std_logic;
				din:	in	std_logic_vector(79 downto 0);
				depth:	in	std_logic_vector( 8 downto 0);
				dout:	out	std_logic_vector(79 downto 0)
			);
		end component;
		
		component L2_engine
			port(
				reset:		in	std_logic;
				clock:		in	std_logic;
				start:		in  std_logic;
				send_fake:	in	std_logic_vector(1 downto 0);
				din:		in	std_logic_vector(79 downto 0);
				sector:		in	std_logic_vector( 3 downto 0);
				octant:		in	std_logic_vector( 2 downto 0);
				cft_frame:	out	std_logic_vector(27 downto 0);	
				cps_frame:	out	std_logic_vector(27 downto 0)	
			);
		end component;


 begin

	-- combinatorial stuff...

		start_engine <= pipeline_dout(0) and L1acc;

		-- actual_depth is what the pipeline is set to.  it is two less than
		-- than the depth variable.  this minimizes the L2 latency.
-- do not see how this can minimize L2 latency. S.X.Wu Oct-12-2004
--		actual_depth(5 downto 0) <= depth(5 downto 0) - 2;


	-- sequential stuff...

--		delay_control: process(clock, reset)  change to sync reset
		delay_control: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				start_delay(7 downto 0)	<= X"00";
				cr_delay(7 downto 0) 	<= X"00";
				fx_delay(7 downto 0) 	<= X"00";
			 else
				start_delay(7 downto 1) <= start_delay(6 downto 0);
				start_delay(0) <= start;

				cr_delay(7 downto 1) <= cr_delay(6 downto 0);
				cr_delay(0) <= cr;

				fx_delay(7 downto 1) <= fx_delay(6 downto 0);
				fx_delay(0) <= fx;
			 end if;
		  end if;
		end process;


	-- instantiate components...

		tick_turn_counter: tick_turn 
		    port map(
				fx					=> fx_delay(7),
				cr					=> cr_delay(7),
				bor					=> start_delay(7),
				l1					=> '1',		-- DFEA is always getting L1 records
				clock				=> clock,
				reset				=> reset,
				tick(7 downto 0)	=> crossing_number(7 downto 0),
				turn(15 downto 0)	=> turn_number(15 downto 0),
				bad_fx				=> bad_fx,
				lock				=> locked
		    );


		L2Packer: L2_packer 
			port map(
				clock 					=> clock,
				reset 					=> reset,
				start 					=> start,
				L2_noTrk					=> L2_noTrk,
				cluster_a(8 downto 0)	=> cluster_a(8 downto 0),		-- format VWwwwCccc
				cluster_b(8 downto 0)	=> cluster_b(8 downto 0),		-- format VWwwwCccc
				crossing(7 downto 0)	=> crossing_number(7 downto 0),
				turn(15 downto 0)		=> turn_number(15 downto 0),
				pm(1 downto 0)			=> pm(1 downto 0),
				iso_track				=> iso_track,
				max_trk(10 downto 0)	=> max_trk(10 downto 0),
				high_trk(10 downto 0)	=> high_trk(10 downto 0),
				med_trk(10 downto 0)	=> med_trk(10 downto 0),
				low_trk(10 downto 0)	=> low_trk(10 downto 0),
				max_cluster(3 downto 0)	=> max_cluster(3 downto 0),
				high_cluster(3 downto 0)=> high_cluster(3 downto 0),
				med_cluster(3 downto 0) => med_cluster(3 downto 0),
				low_cluster(3 downto 0) => low_cluster(3 downto 0),
				bor						=> packer_bor,
				dout(79 downto 0)		=> packer_dout(79 downto 0)
			);
	
		L2Pipeline: L2_pipeline
			port map(
				reset				=> reset,
				clock				=> clock,
				bor  				=> packer_bor,
				din(79 downto 0)	=> packer_dout(79 downto 0),
--				depth(5 downto 0)	=> actual_depth(5 downto 0),
				depth	=> depth,
				dout(79 downto 0)	=> pipeline_dout(79 downto 0)
			);

		L2Engine: L2_engine
			port map(
				reset					=> reset,
				clock 					=> clock,
				start					=> start_engine,
				send_fake(1 downto 0)	=> send_fake(1 downto 0),
				din(79 downto 0)		=> pipeline_dout(79 downto 0),
				sector(3 downto 0)		=> sector(3 downto 0),
				octant(2 downto 0)		=> octant(2 downto 0),
				cft_frame(27 downto 0) 	=> cft_frame(27 downto 0),
				cps_frame(27 downto 0)	=> cps_frame(27 downto 0)
			);

end l2_top_arch;
