-- L2_trksort.vhd
-- part of the L2CFT engine track sorter
-- jamieson olsen <jamieson@fnal.gov>
-- 26 Sept 2002

-- sorts and zero compresses L2CFT tracks as they fall out of the pipeline.  The result
-- sequence of tracks should be sorted in order of decreasing Pt and contain no gaps.
-- assumes that tracks in each Pt bin are sorted already and contain no gaps.  up to 
-- 6 tracks per pt bin.  all tracks are dumped into this module in parallel, in 
-- 6 clock cycles.  up to 24 sorted tracks are read out with a latency of 2 clock cycles.  

-- see engineering note 2002-09-19a.

-- input bus format is: Valid, Curve, ExtPt[2..0], Phi[5..0], ClusterMatch, ClusterNumber[2..0]

-- 13 May 2003:  consistant curve 1 = CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2_trksort is
	port(
		clock:	in	std_logic;
		reset:	in	std_logic;
		enable:	in	std_logic;
		max:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
		high:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
		med:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
		low:	in	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn

		tvalid:	out	std_logic;						-- valid track
		curve:	out	std_logic;		
		pt:		out	std_logic_vector(1 downto 0);
		extpt:	out	std_logic_vector(2 downto 0);	-- extended pt
		phi:	out	std_logic_vector(5 downto 0);	
		cm:		out	std_logic;						-- there is a cluster match
		cnum:	out	std_logic_vector(2 downto 0)	-- matching cluster index
    );
end L2_trksort;


architecture L2_trksort_arch of L2_trksort is

	-- define registers...
	
		signal max_reg:		std_logic_vector(14 downto 0);
		signal dout1_reg:	std_logic_vector(16 downto 0);
		signal dout2_reg:	std_logic_vector(16 downto 0);
--		signal count_reg:	std_logic_vector( 5 downto 0);

	-- define signals

		signal enable_dl:	std_logic;
		signal flush:		std_logic;
		signal mux:			std_logic_vector(16 downto 0);
		signal read:		std_logic_vector( 3 downto 0);
		signal high_dout:	std_logic_vector(14 downto 0);
		signal med_dout:	std_logic_vector(14 downto 0);
		signal low_dout:	std_logic_vector(14 downto 0);

	-- declare components...

		component L2CFT_fe
			port(
				clock:	in	std_logic;
				reset:	in	std_logic;
				write:	in	std_logic;						-- active high
				read:	in	std_logic;						-- active high
				flush:	in	std_logic;						-- active high
				din: 	in 	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
				dout: 	out std_logic_vector(14 downto 0) 	-- format VCEeePpppppMNnn
		    );
		end component;
		
 begin

	-- combinatorial stuff...

		tvalid 				<= dout2_reg(16);
		curve				<= dout2_reg(15);
		pt(1 downto 0) 		<= dout2_reg(14 downto 13);
		extpt(2 downto 0)	<= dout2_reg(12 downto 10);
		phi(5 downto 0)		<= dout2_reg( 9 downto 4);
		cm					<= dout2_reg(3);
		cnum(2 downto 0)	<= dout2_reg(2 downto 0);



		-- select which track bus to pass on the output
			read(3 downto 0) <= "0001" when (max_reg(14)='1') else		
								"0010" when (max_reg(14)='0' and high_dout(14)='1') else
								"0100" when (max_reg(14)='0' and high_dout(14)='0' and med_dout(14)='1') else
								"1000" when (max_reg(14)='0' and high_dout(14)='0' and med_dout(14)='0' and low_dout(14)='1') else
								"0000";

		-- do the actual muxing: V, C                       PT    ExtPt, Phi, ClusterMatch, and ClusterNumber
			mux(16 downto 0) <= (max_reg(14 downto 13)   & "00" & max_reg(12 downto 0))  when read(3 downto 0) ="0001" else
								(high_dout(14 downto 13) & "01" & high_dout(12 downto 0)) when read(3 downto 0)="0010" else
								(med_dout(14 downto 13)  & "10" & med_dout(12 downto 0))  when read(3 downto 0)="0100" else
								(low_dout(14 downto 13)  & "11" & low_dout(12 downto 0))  when read(3 downto 0)="1000" else
								"00000000000000000";

		-- after the tracks have been read out, flush out the front end RAMs.
--			flush <= '1' when (count_reg(5 downto 0)<"111101" and count_reg(5 downto 0)>"101100") else '0';
-- flush was too late, can interfere with next L1 03-11-2005 S.X.Wu
-- Now flush is only used to reset l2cft_fe to reset rptr_reg. this also avoids similar problem as
-- in muon_fe.vhd
			flush <= enable and not enable_dl;

	-- sequential stuff...

--		process(clock, reset)	change to sync reset  Aug-06-04
		process(clock)
 		 begin
			if rising_edge(clock) then
			 if (reset='0') then
			   enable_dl <= '0';
				max_reg(14 downto 0)  <= "000000000000000";
--				count_reg(5 downto 0) <= "111111";
				dout1_reg(16 downto 0) <= "00000000000000000";
				dout2_reg(16 downto 0) <= "00000000000000000";
			 else
			   enable_dl <= enable;
				if (enable='1') then
					max_reg(14 downto 0) <= max(14 downto 0);
				else
					max_reg(14 downto 0) <= "000000000000000";
				end if;

--				if (enable='1') then
--					count_reg(5 downto 0) <= "000000";
--				elsif (count_reg(5 downto 0) /= "111111") then
--					count_reg(5 downto 0) <= count_reg(5 downto 0) + 1;
--				end if;

				dout1_reg(16 downto 0) <= mux(16 downto 0);
				dout2_reg(16 downto 0) <= dout1_reg(16 downto 0);

			 end if;
		  end if;
		end process;
	
	-- instantiate components...

		L2HighFifo: L2CFT_fe
			port map(
				clock	=> clock,
				reset	=> reset,
				write	=> enable,
				read	=> read(1),
				flush	=> flush,
				din(14 downto 0) => high(14 downto 0),
				dout(14 downto 0) => high_dout(14 downto 0)
		    );

		L2MedFifo: L2CFT_fe
			port map(
				clock	=> clock,
				reset	=> reset,
				write	=> enable,
				read	=> read(2),
				flush	=> flush,
				din(14 downto 0) => med(14 downto 0),
				dout(14 downto 0) => med_dout(14 downto 0)
		    );

		L2LowFifo: L2CFT_fe
			port map(
				clock	=> clock,
				reset	=> reset,
				write	=> enable,
				read	=> read(3),
				flush	=> flush,
				din(14 downto 0) => low(14 downto 0),
				dout(14 downto 0) => low_dout(14 downto 0)
		    );
		
end L2_trksort_arch;

