-- L2CFT_fe.vhd
-- part of the L2CFT engine track sorter
-- jamieson olsen <jamieson@fnal.gov>
-- 19 Sept 2002

-- bus format is: Valid, Curve, ExtPt[2..0], Phi[5..0], ClusterMatch, ClusterNumber[2..0]

-- operation: write should be asserted for 6 clock cyles when tracks are being clocked in.
-- this stores the tracks into memory locations 0-5.  after the first track is clocked in, the output
-- will reflect what's stored in the first position of the ram.  asserting read will 
-- advance the read pointer to the next track (if it exists)

-- the write signal happens only when there's a L1accept, which is infrequently.  After all data 
-- has been read out, assert FLUSH for 16 clock cycles to completely empty out the ram and 
-- reset the read pointer back to zero.  now it will be ready for the next batch of tracks.

-- 13 May 2003:  consistant curve 1 = CCW

-- 3/10/2005 flush only resets rptr_reg at the first clock cycle after enable goes high S. X. Wu
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2CFT_fe is
	port(
		clock:	in	std_logic;
		reset:	in	std_logic;
		write:	in	std_logic;						-- active high
		read:	in	std_logic;						-- active high
		flush:	in	std_logic;						-- active high
		din: 	in 	std_logic_vector(14 downto 0);	-- format VCEeePpppppMNnn
		dout: 	out std_logic_vector(14 downto 0) 	-- format VCEeePpppppMNnn
    );
end L2CFT_fe;


architecture L2CFT_fe_arch of L2CFT_fe is

	-- define registers...
	
		signal wptr_reg:	std_logic_vector(3 downto 0);	-- write pointer
		signal rptr_reg:	std_logic_vector(3 downto 0);	-- read pointer

	-- define signals

		signal data:		std_logic_vector(14 downto 0);	-- data to be written
		signal we:			std_logic;

	-- declare components...

		component ram15x16d 
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector( 3 downto 0);	-- write port address
				din:		in	std_logic_vector(14 downto 0);
				addrb:		in	std_logic_vector( 3 downto 0);	-- read port address
				dout:		out	std_logic_vector(14 downto 0)	-- read data
			);
		end component;
		
 begin

	-- combinatorial stuff...

--		data(14 downto 0) <= "000000000000000" when (flush='1') else din(14 downto 0);
		data(14 downto 0) <= din(14 downto 0);

--		we <= write or flush;
		we <= write;

	-- sequential stuff...
	
--		pointers: process(clock, reset)  change to sync reset
		pointers: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then	
				wptr_reg(3 downto 0)	<= "0000";
				rptr_reg(3 downto 0)	<= "0000";
			 else
--				if (write='1' or flush='1') then
				if (write='1') then
					wptr_reg(3 downto 0) <= wptr_reg(3 downto 0) + 1;
				else
					wptr_reg(3 downto 0) <= "0000";
				end if;
					
--				if (read='1') then
--					rptr_reg(3 downto 0) <= rptr_reg(3 downto 0) + 1;
--				elsif (flush='1') then
--					rptr_reg(3 downto 0) <= "0000";
--				end if;
				if (flush='1') then
					rptr_reg(3 downto 0) <= "0000";
				elsif (read='1') then
					rptr_reg(3 downto 0) <= rptr_reg(3 downto 0) + 1;
				end if;

			 end if;
		  end if;
		end process;

	-- instantiate components...

		l2_track_fifo: ram15x16d 
		 	port map(
				wclk				=> clock,
				we					=> we,
				addra(3 downto 0)	=> wptr_reg(3 downto 0),
				din(14 downto 0)	=> data(14 downto 0),
				addrb(3 downto 0)	=> rptr_reg(3 downto 0),
				dout(14 downto 0)	=> dout(14 downto 0)
			);
		
end L2CFT_fe_arch;

