--	mobo_int.vhd
-- 	jamieson olsen <jamieson@fnal.gov>

--  DFE Motherboard Interface (bridge logic)

-- 	The DFE motherboard writes to this module using the depth_clk, depth_dat and depth_rst lines
--  10 bits are shifted into this device each time the DFEC writes to the DFEM backplane registers
--	at offset 5-8. 

--  depth_clk	    /    /	   /     /     /     /     /     /     /     /     /     /     /
--	depth_dat	XXXXXX< S1 >< S0 >< D7 >< D6 >< D5 >< D4 >< D3 >< D2 >< D1 >< D0 >XXXXXXXX
--	depth_rst	______/----------------------------------------------------------\________

-- Where S1 and S0 specify which DFEM backplane register was written:  00 = reg 5, 01=reg 6, etc.
-- offset 5 = page register
-- 0ffset 6 = L3 pipeline depth (0-63)
-- offset 7 = home sector register
-- offset 8 = bus control register (bit mapped)


-- *** recent revision history ***
--  23 Oct 2002:	add output to control fake CTOC L2 records
--  05 Mar 2003:	enhanced communication with DFEM U49.  fake L2 signals are now 2 bits.
--  07 Mar 2003:	add history registers, FX and SG checking, etc.
--  13 Mar 2003:	remove circuit that checks that the period of FX is correct.  replace that 
--					history bit with bad_fx which comes from the new tick_turn counter in the 
--					L2 functional block.  Add CFT_RESET history bit and re-arrange the status bits
--					in the pages.

library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;

entity mobo_int is
	port(
		reset:		in	std_logic;						-- active low, async.
		clock:		in	std_logic;						-- master clock, 53MHz
-- bus_dir read = '1', write = '0'
      wr_mobo:	   in std_logic;
      rd_mobo:	   in std_logic;
		addr:       in std_logic_vector(9 downto 0);					
		w_data:     in std_logic_vector(15 downto 0);					
		r_data:     out std_logic_vector(15 downto 0);					
		fx:			in	std_logic;						-- First Crossing
		sg:			in	std_logic;						-- Sync Gap
		start:		in	std_logic;						-- beginning of crossing marker

		muon_noTrk:		out	std_logic;						-- disables muon track output
		L1_noTrk:		out	std_logic;						-- disables L1 track output
		L2_noTrk:		out	std_logic;						-- disables L2 track output
		sg_err:			out	std_logic;						-- set to force fake muon track in bunch 10.
		muon_fake:		out	std_logic;						-- set to force fake muon track in bunch 10.
		muon_patt:		out	std_logic;						-- set to force muon pattern
		new_muon:		out	std_logic;						-- enable run2a muon match and isolation bits
		ctoc_patt:		out std_logic;						-- set to force CTOC ouput pattern
		ctoc_fake_sel:	out	std_logic_vector(1 downto 0);	-- select fake L1 CTOC patterns (00=normal)
		ctoc_zero:		out	std_logic;						-- zero out the L1/L2 bus to CTOC
		ctoc_L2_fake:	out	std_logic_vector(1 downto 0);	-- send fake L2 records in response to L1ACCEPT
--		mem_depth:		out	std_logic_vector(6 downto 0);	-- L2 pipeline depth (in crossings)
		rel_sect:		out	std_logic_vector(3 downto 0);	-- relative sector within a super-sector (0-15)
		octant:			out	std_logic_vector(2 downto 0)	-- octant 0-7
	);	
end mobo_int;

architecture mobo_int_arch of mobo_int is

	-- declare registers...

--		signal depth_reg:	std_logic_vector(6 downto 0);
		signal sector_reg:	std_logic_vector(15 downto 0);
		signal bus_reg:		std_logic_vector(11 downto 0);
		signal sg_err_reg: 	std_logic;
		signal sg_count:	std_logic_vector( 4 downto 0);


	-- declare signals...


-- readback register	
signal mobo:	std_logic_vector(15 downto 0);
signal addr0_q : std_logic;
begin

	-- combinatorial stuff...

			muon_noTrk <= bus_reg(9);	-- disables muon track
			L1_noTrk <= bus_reg(10);	-- disables L1 track
			L2_noTrk <= bus_reg(11);	-- disables L2 track
		-- control the L1 muon output bus...

			muon_patt <= bus_reg(0);	-- johnson pattern
			muon_fake <= bus_reg(1);	-- insert max track in crossing 10

		-- control the 28-bit output bus going to CTOC/STOV/STSX...

			ctoc_zero <= bus_reg(2);	-- completely turn off the output bus
			ctoc_patt <= bus_reg(3);	-- drive 28-bit johnson pattern (not really used anymore).

		-- control what kinds of L1 records go out to the CTOC/STOV/STSX.
		-- the meaning of these bits is determined in level_one_format.vhd

			ctoc_fake_sel(1 downto 0) <= bus_reg(5 downto 4);	

		-- send fake L2CFT/L2CPS records on the CTOC output in response to a L1ACCEPT.

			ctoc_L2_fake(1 downto 0) <= bus_reg(7 downto 6);

		-- drive the L2 pipeline depth out to the rest of U1...
			new_muon <= bus_reg(8);
--			mem_depth(6 downto 0)	<= depth_reg(6 downto 0);

		-- octant 0 contains trigger sectors 0-9, etc.  generated by excel
		-- spreadsheet 2002-02-07a...

		octant(2 downto 0) <= 	"000" when (sector_reg(6 downto 0)="0000000") else
								"000" when (sector_reg(6 downto 0)="0000001") else
								"000" when (sector_reg(6 downto 0)="0000010") else
								"000" when (sector_reg(6 downto 0)="0000011") else
								"000" when (sector_reg(6 downto 0)="0000100") else
								"000" when (sector_reg(6 downto 0)="0000101") else
								"000" when (sector_reg(6 downto 0)="0000110") else
								"000" when (sector_reg(6 downto 0)="0000111") else
								"000" when (sector_reg(6 downto 0)="0001000") else
								"000" when (sector_reg(6 downto 0)="0001001") else
								"001" when (sector_reg(6 downto 0)="0001010") else
								"001" when (sector_reg(6 downto 0)="0001011") else
								"001" when (sector_reg(6 downto 0)="0001100") else
								"001" when (sector_reg(6 downto 0)="0001101") else
								"001" when (sector_reg(6 downto 0)="0001110") else
								"001" when (sector_reg(6 downto 0)="0001111") else
								"001" when (sector_reg(6 downto 0)="0010000") else
								"001" when (sector_reg(6 downto 0)="0010001") else
								"001" when (sector_reg(6 downto 0)="0010010") else
								"001" when (sector_reg(6 downto 0)="0010011") else
								"010" when (sector_reg(6 downto 0)="0010100") else
								"010" when (sector_reg(6 downto 0)="0010101") else
								"010" when (sector_reg(6 downto 0)="0010110") else
								"010" when (sector_reg(6 downto 0)="0010111") else
								"010" when (sector_reg(6 downto 0)="0011000") else
								"010" when (sector_reg(6 downto 0)="0011001") else
								"010" when (sector_reg(6 downto 0)="0011010") else
								"010" when (sector_reg(6 downto 0)="0011011") else
								"010" when (sector_reg(6 downto 0)="0011100") else
								"010" when (sector_reg(6 downto 0)="0011101") else
								"011" when (sector_reg(6 downto 0)="0011110") else
								"011" when (sector_reg(6 downto 0)="0011111") else
								"011" when (sector_reg(6 downto 0)="0100000") else
								"011" when (sector_reg(6 downto 0)="0100001") else
								"011" when (sector_reg(6 downto 0)="0100010") else
								"011" when (sector_reg(6 downto 0)="0100011") else
								"011" when (sector_reg(6 downto 0)="0100100") else
								"011" when (sector_reg(6 downto 0)="0100101") else
								"011" when (sector_reg(6 downto 0)="0100110") else
								"011" when (sector_reg(6 downto 0)="0100111") else
								"100" when (sector_reg(6 downto 0)="0101000") else
								"100" when (sector_reg(6 downto 0)="0101001") else
								"100" when (sector_reg(6 downto 0)="0101010") else
								"100" when (sector_reg(6 downto 0)="0101011") else
								"100" when (sector_reg(6 downto 0)="0101100") else
								"100" when (sector_reg(6 downto 0)="0101101") else
								"100" when (sector_reg(6 downto 0)="0101110") else
								"100" when (sector_reg(6 downto 0)="0101111") else
								"100" when (sector_reg(6 downto 0)="0110000") else
								"100" when (sector_reg(6 downto 0)="0110001") else
								"101" when (sector_reg(6 downto 0)="0110010") else
								"101" when (sector_reg(6 downto 0)="0110011") else
								"101" when (sector_reg(6 downto 0)="0110100") else
								"101" when (sector_reg(6 downto 0)="0110101") else
								"101" when (sector_reg(6 downto 0)="0110110") else
								"101" when (sector_reg(6 downto 0)="0110111") else
								"101" when (sector_reg(6 downto 0)="0111000") else
								"101" when (sector_reg(6 downto 0)="0111001") else
								"101" when (sector_reg(6 downto 0)="0111010") else
								"101" when (sector_reg(6 downto 0)="0111011") else
								"110" when (sector_reg(6 downto 0)="0111100") else
								"110" when (sector_reg(6 downto 0)="0111101") else
								"110" when (sector_reg(6 downto 0)="0111110") else
								"110" when (sector_reg(6 downto 0)="0111111") else
								"110" when (sector_reg(6 downto 0)="1000000") else
								"110" when (sector_reg(6 downto 0)="1000001") else
								"110" when (sector_reg(6 downto 0)="1000010") else
								"110" when (sector_reg(6 downto 0)="1000011") else
								"110" when (sector_reg(6 downto 0)="1000100") else
								"110" when (sector_reg(6 downto 0)="1000101") else
								"111";

		rel_sect(3 downto 0) <=	"0001" when (sector_reg(6 downto 0)="0000001") else
								"0010" when (sector_reg(6 downto 0)="0000010") else
								"0011" when (sector_reg(6 downto 0)="0000011") else
								"0100" when (sector_reg(6 downto 0)="0000100") else
								"0101" when (sector_reg(6 downto 0)="0000101") else
								"0110" when (sector_reg(6 downto 0)="0000110") else
								"0111" when (sector_reg(6 downto 0)="0000111") else
								"1000" when (sector_reg(6 downto 0)="0001000") else
								"1001" when (sector_reg(6 downto 0)="0001001") else
								"0001" when (sector_reg(6 downto 0)="0001011") else
								"0010" when (sector_reg(6 downto 0)="0001100") else
								"0011" when (sector_reg(6 downto 0)="0001101") else
								"0100" when (sector_reg(6 downto 0)="0001110") else
								"0101" when (sector_reg(6 downto 0)="0001111") else
								"0110" when (sector_reg(6 downto 0)="0010000") else
								"0111" when (sector_reg(6 downto 0)="0010001") else
								"1000" when (sector_reg(6 downto 0)="0010010") else
								"1001" when (sector_reg(6 downto 0)="0010011") else
								"0001" when (sector_reg(6 downto 0)="0010101") else
								"0010" when (sector_reg(6 downto 0)="0010110") else
								"0011" when (sector_reg(6 downto 0)="0010111") else
								"0100" when (sector_reg(6 downto 0)="0011000") else
								"0101" when (sector_reg(6 downto 0)="0011001") else
								"0110" when (sector_reg(6 downto 0)="0011010") else
								"0111" when (sector_reg(6 downto 0)="0011011") else
								"1000" when (sector_reg(6 downto 0)="0011100") else
								"1001" when (sector_reg(6 downto 0)="0011101") else
								"0001" when (sector_reg(6 downto 0)="0011111") else
								"0010" when (sector_reg(6 downto 0)="0100000") else
								"0011" when (sector_reg(6 downto 0)="0100001") else
								"0100" when (sector_reg(6 downto 0)="0100010") else
								"0101" when (sector_reg(6 downto 0)="0100011") else
								"0110" when (sector_reg(6 downto 0)="0100100") else
								"0111" when (sector_reg(6 downto 0)="0100101") else
								"1000" when (sector_reg(6 downto 0)="0100110") else
								"1001" when (sector_reg(6 downto 0)="0100111") else
								"0001" when (sector_reg(6 downto 0)="0101001") else
								"0010" when (sector_reg(6 downto 0)="0101010") else
								"0011" when (sector_reg(6 downto 0)="0101011") else
								"0100" when (sector_reg(6 downto 0)="0101100") else
								"0101" when (sector_reg(6 downto 0)="0101101") else
								"0110" when (sector_reg(6 downto 0)="0101110") else
								"0111" when (sector_reg(6 downto 0)="0101111") else
								"1000" when (sector_reg(6 downto 0)="0110000") else
								"1001" when (sector_reg(6 downto 0)="0110001") else
								"0001" when (sector_reg(6 downto 0)="0110011") else
								"0010" when (sector_reg(6 downto 0)="0110100") else
								"0011" when (sector_reg(6 downto 0)="0110101") else
								"0100" when (sector_reg(6 downto 0)="0110110") else
								"0101" when (sector_reg(6 downto 0)="0110111") else
								"0110" when (sector_reg(6 downto 0)="0111000") else
								"0111" when (sector_reg(6 downto 0)="0111001") else
								"1000" when (sector_reg(6 downto 0)="0111010") else
								"1001" when (sector_reg(6 downto 0)="0111011") else
								"0001" when (sector_reg(6 downto 0)="0111101") else
								"0010" when (sector_reg(6 downto 0)="0111110") else
								"0011" when (sector_reg(6 downto 0)="0111111") else
								"0100" when (sector_reg(6 downto 0)="1000000") else
								"0101" when (sector_reg(6 downto 0)="1000001") else
								"0110" when (sector_reg(6 downto 0)="1000010") else
								"0111" when (sector_reg(6 downto 0)="1000011") else
								"1000" when (sector_reg(6 downto 0)="1000100") else
								"1001" when (sector_reg(6 downto 0)="1000101") else
								"0001" when (sector_reg(6 downto 0)="1000111") else
								"0010" when (sector_reg(6 downto 0)="1001000") else
								"0011" when (sector_reg(6 downto 0)="1001001") else
								"0100" when (sector_reg(6 downto 0)="1001010") else
								"0101" when (sector_reg(6 downto 0)="1001011") else
								"0110" when (sector_reg(6 downto 0)="1001100") else
								"0111" when (sector_reg(6 downto 0)="1001101") else
								"1000" when (sector_reg(6 downto 0)="1001110") else
								"1001" when (sector_reg(6 downto 0)="1001111") else
								"0000";
r_data <= mobo when rd_mobo = '1' else (others => 'Z');
	-- sequential stuff...
process(clock)
begin
  if(clock'event and clock = '1')then
    addr0_q <= addr(0);
    if(reset = '0')then
	   sector_reg <= x"0000";
	   bus_reg <= x"000";
	 elsif(wr_mobo = '1')then
	   if(addr0_q = '0')then
	     sector_reg <= w_data;
		else
	     bus_reg <= w_data(11 downto 0);
		end if;
	 end if;
	 if(addr(0) = '0')then
	   mobo <= sector_reg;
	 else
	   mobo <= x"0" & bus_reg;
	 end if;
  end if;
end process;
		-- check that the length of sync_gap agrees with
		-- the expected value: 17 crossings
		-- start is asserted every crossing, whether the system is in L1 or L2 mode.

--			ctrl_checker_proc: process(clock, reset)  change to sync reset
			ctrl_checker_proc: process(clock)
			 begin
				if rising_edge(clock) then
				 if (reset='0') then
					sg_err <= '0';
					sg_count(4 downto 0) <= "00000";
				 else

					if (start='1') then
						if (sg='1') then
							sg_count <= sg_count + 1;
						end if;
					
						if (fx='1') then
							if (sg_count(4 downto 0) /= "10001") then
								sg_err <= '1';
							else
								sg_err <= '0';
							end if;
							sg_count(4 downto 0) <= "00000";
						end if;

					 end if;
				  end if;
				end if;
			end process;
end mobo_int_arch;
