-- cluster_finder_b.vhd
-- CPS axial cluster finder
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 20 march 2002: created, functionally simulated.

-- CPS index 15 is closest to the center of the home sector, this bit has the highest 
-- priority.  Find clusters starting in the center and working outwards in the negative
-- phi direction.  see eng. note 2002-01-22a for details.

-- cluster centroid is calculated as (inner_index + outer_index)/2
-- cluster width is calculated as (inner_index - outer_index) + 1

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity cluster_finder_b is
 port(
    inner_edge:	in  std_logic_vector(15 downto 0);
    outer_edge:	in  std_logic_vector(15 downto 0);
	clock:		in	std_logic;
    load:       in  std_logic;
    reset:      in  std_logic;                       -- async reset active low

	valid:		out	std_logic;
	width:		out	std_logic_vector(3 downto 0);
	center:		out std_logic_vector(3 downto 0)
    );
end cluster_finder_b;


architecture cluster_finder_b_arch of cluster_finder_b is

	-- define registers...

		signal inner_index_reg: 	std_logic_vector(3 downto 0);
		signal outer_index_reg: 	std_logic_vector(3 downto 0);
		signal center_reg:			std_logic_vector(3 downto 0);
		signal width_reg:			std_logic_vector(3 downto 0);
		signal valid_reg:			std_logic_vector(1 downto 0);

	-- define signals...

		signal inner_onehot_index:	std_logic_vector(15 downto 0);
		signal outer_onehot_index:	std_logic_vector(15 downto 0);
		signal inner_index:			std_logic_vector( 3 downto 0);
		signal outer_index:			std_logic_vector( 3 downto 0);
		signal temp_valid:			std_logic;
		signal temp_center:			std_logic_vector( 4 downto 0);
		signal temp_width:			std_logic_vector( 3 downto 0);

	-- declare components...

		component sprior16b 
		 port(
		    din:        in  std_logic_vector(15 downto 0);
		    clock:      in  std_logic;
		    load:       in  std_logic;                       -- sync load active high
		    reset:      in  std_logic;                       -- async reset active low
		    dout:       out std_logic_vector(15 downto 0)
		    );
		end component;

		component encoder16
		 port(
			    d:	in  std_logic_vector(15 downto 0);
				q:	out	std_logic_vector( 3 downto 0)
		    );
		end component;

 begin

    -- combinatorial stuff...
--    3/11/2005 overflow handling S. X. Wu
--		temp_width(3 downto 0)	<= (inner_index_reg(3 downto 0) - outer_index_reg(3 downto 0)) + 1;
		temp_width(3 downto 0)	<= (inner_index_reg(3 downto 0) - outer_index_reg(3 downto 0));

		temp_center(4 downto 0)	<= ('0'& outer_index_reg(3 downto 0)) + ('0' & inner_index_reg(3 downto 0));

		temp_valid <=  (outer_onehot_index( 0) or  outer_onehot_index( 1) or 
						outer_onehot_index( 2) or  outer_onehot_index( 3) or 
						outer_onehot_index( 4) or  outer_onehot_index( 5) or 
						outer_onehot_index( 6) or  outer_onehot_index( 7) or 
						outer_onehot_index( 8) or  outer_onehot_index( 9) or 
						outer_onehot_index(10) or  outer_onehot_index(11) or 
						outer_onehot_index(12) or  outer_onehot_index(13) or 
						outer_onehot_index(14) or  outer_onehot_index(15)) and 

					   (inner_onehot_index( 0) or  inner_onehot_index( 1) or 
						inner_onehot_index( 2) or  inner_onehot_index( 3) or 
						inner_onehot_index( 4) or  inner_onehot_index( 5) or 
						inner_onehot_index( 6) or  inner_onehot_index( 7) or 
						inner_onehot_index( 8) or  inner_onehot_index( 9) or 
						inner_onehot_index(10) or  inner_onehot_index(11) or 
						inner_onehot_index(12) or  inner_onehot_index(13) or 
						inner_onehot_index(14) or  inner_onehot_index(15));

		width(3 downto 0)	<= width_reg(3 downto 0);
		valid				<= valid_reg(1);
		center(3 downto 0)	<= center_reg(3 downto 0);



    -- sequential stuff...

--	    process(clock, reset)	 change to sync reset Aug-08-04
	    process(clock)
	     begin

	       if rising_edge(clock) then
	        if (reset='0') then

				inner_index_reg(3 downto 0)	<= "0000";
				outer_index_reg(3 downto 0)	<= "0000";
				center_reg(3 downto 0)		<= "0000";
				width_reg(3 downto 0)		<= "0000";
				valid_reg(1 downto 0)		<= "00";
			  else
				inner_index_reg(3 downto 0)	<= inner_index(3 downto 0);
				outer_index_reg(3 downto 0)	<= outer_index(3 downto 0);
--				width_reg(3 downto 0)		<= temp_width(3 downto 0);
--    3/11/2005 overflow handling S. X. Wu
				if(temp_width = "1111")then
				  width_reg(3 downto 0)		<= temp_width(3 downto 0);
				else
				  width_reg(3 downto 0)		<= temp_width(3 downto 0) + 1;
				end if;
				center_reg(3 downto 0)		<= temp_center(4 downto 1);
				valid_reg(0)				<= temp_valid;
				valid_reg(1)				<= valid_reg(0);

	        end if;
			end if;
	    end process;

	-- instantiate components...

		inner_edge_detector: sprior16b 
		 	port map(
			    din(15 downto 0)	=> inner_edge(15 downto 0),
			    clock				=> clock,
			    load				=> load,
			    reset				=> reset,
			    dout(15 downto 0)	=> inner_onehot_index(15 downto 0)
		    );

		outer_edge_detector: sprior16b 
		 	port map(
			    din(15 downto 0)	=> outer_edge(15 downto 0),
			    clock				=> clock,
			    load				=> load,
			    reset				=> reset,
			    dout(15 downto 0)	=> outer_onehot_index(15 downto 0)
		    );

		inner_encoder: encoder16
		 	port map(
			    d(15 downto 0)		=> inner_onehot_index(15 downto 0),
				q( 3 downto 0)		=> inner_index(3 downto 0)
		    );

		outer_encoder: encoder16
		 	port map(
			    d(15 downto 0)		=> outer_onehot_index(15 downto 0),
				q( 3 downto 0)		=> outer_index(3 downto 0)
		    );

end cluster_finder_b_arch;
